from {{ func_name }} import {{ func_class }}
from stonewave.sql.udtfs.test_utility import (
    check_expected_parameters_list,
    eval_table_function_test,
)
from tests.supported_signature_list import supported_signature_list
import pytest


def _execute_function(args):
    func_expected_params = supported_signature_list()
    check_expected_parameters_list(func_expected_params)

    batch_iterator = eval_table_function_test(
        {{ func_class }}, args, func_expected_params
    )
    return list(batch_iterator)


def _assert_add_two(num1, num2, expected_result):
    args = [num1, num2]
    batches = _execute_function(args)
    assert len(batches) == 1
    batch = batches[0]
    assert batch.num_rows == 1
    assert batch.num_columns == 1
    assert batch.schema.names == ["add_result"]
    results = batch[batch.schema.get_field_index("add_result")][0].as_py()
    # write_row method will make all results as string type
    assert str(results) == str(expected_result)


def test_add_two():
    _assert_add_two(3, 5, 8)
    _assert_add_two(10, 12, 22)


from stonewave.sql.udtfs.constants import ParameterDataType as pt
import toml
from pathlib import Path


def _param_to_param_data_type(param):
    # this mandates ParameterDataType to name its attributes the same as the values in info.toml
    return getattr(pt, param)


def _toml_sig_to_params_list(params):
    return [_param_to_param_data_type(param) for param in params]


def supported_signature_list():
    info_toml = Path(__file__).parent.parent / "{{ func_name }}" / "info.toml"
    with open(info_toml, "r") as f:
        info = toml.load(f)
        sigs = [_toml_sig_to_params_list(sig) for sig in info["signature_list"]]
        return sigs



from pathlib import Path, PurePath
import os
from jinja2 import Environment, PackageLoader
import json
import os
import shutil


def write_file(output_dir, file_name, content):
    with open(output_dir / file_name, "w") as file:
        file.write(content)


def to_camel_case(snake_str):
    components = snake_str.split("_")
    return "".join(x.title() for x in components)


def write_file(output_dir, file_name, content):
    with open(output_dir / file_name, "w") as file:
        file.write(content)


def _generated_file_name(file):
    return file.replace(".template", "")


def _package_name(template_dir_path):
    parent = str(template_dir_path.parent)
    # single path segment
    if parent == ".":
        return f"eggfarm.templates"
    else:
        return f"eggfarm.templates.{parent.replace('/', '.')}"


def _generate_file_with_template(output_dir, template_dir, file, function_name):
    camel_cased_function_name = to_camel_case(function_name)
    function_class = f"{camel_cased_function_name}Function"

    template_dir_path = PurePath(template_dir)
    if template_dir_path.name:
        package_name = _package_name(template_dir_path)
        env = Environment(loader=PackageLoader(package_name, template_dir_path.name))
    else:
        env = Environment(loader=PackageLoader("eggfarm", "templates"))

    template = env.get_template(file)
    content = template.render(func_name=function_name, func_class=function_class)

    if template_dir == ".":
        file_dir = output_dir
    else:
        file_dir = os.path.join(output_dir, template_dir)

    write_file(Path(file_dir), _generated_file_name(file), content)


# the eggfarm.templates.__init__.py is just to make jinja2 happy to load template
def _is_template(file):
    return ".template" in file


def _check_output_dir(output_dir, default_dir):
    if output_dir is None:
        output_dir = default_dir

    output_dir_path = Path(os.path.join(output_dir))
    if output_dir_path.exists():
        if output_dir_path.is_file():
            raise FileExistsError(f"{output_dir} is a file, please specify a directory as output directory")
        elif any(os.scandir(output_dir_path)):
            raise FileExistsError(f"{output_dir} is not empty, please use an empty directory as output directory")
    else:
        output_dir_path.mkdir(parents=True, exist_ok=True)
    return output_dir


def generate_skeleton(args):
    function_name = args["function_name"]
    output_dir = _check_output_dir(args.get("output_dir"), function_name)

    print(f"generating table function skeleton for {function_name} under directory {output_dir}")
    templates_dir = os.path.join(os.path.dirname(os.path.dirname(__file__)), "eggfarm/templates")

    for root, dirs, files in os.walk(templates_dir):
        if os.path.basename(root) == "__pycache__":
            continue
        for dir in dirs:
            if dir == "__pycache__":
                continue
            Path(os.path.join(output_dir, dir)).mkdir(parents=True, exist_ok=True)

        for file in files:
            if _is_template(file):
                template_dir = os.path.relpath(root, templates_dir)
                _generate_file_with_template(output_dir, template_dir, file, function_name)

    os.rename(os.path.join(output_dir, "func"), os.path.join(output_dir, function_name))



import importlib.metadata

package_metadada = importlib.metadata.metadata("{{ func_name }}")
# info from pyproject.toml's `version` and `description`
VERSION = package_metadada.get("Version")


def {{ func_name }}_version():
    return VERSION



import sys
import os
from eggfarm.table_func_generator import generate_skeleton

import argparse
from distutils.util import strtobool

import importlib.metadata

package_metadada = importlib.metadata.metadata("eggfarm")
# info from pyproject.toml's `version` and `description`
EGGFARM_VERSION = package_metadada.get("Version")
EGGFARM_SUMMARY = package_metadada.get("Summary")


def _eggfarm_parser():
    parser = argparse.ArgumentParser(prog="eggfarm")
    parser.add_argument(
        "--version",
        action="version",
        version=f"%(prog)s {EGGFARM_VERSION} [{EGGFARM_SUMMARY}]",
        help="show version number",
    )
    return parser


def _new_command_parser(sub_parsers):
    new_parser = sub_parsers.add_parser(
        "new",
        help="generate python table function skeleton",
    )

    new_parser.add_argument(
        "function_name",
        default=False,
        type=str,
        help="the name for the new table function, use `_` to connect multiple words (snake_case), e.g. `my_table_func`",
    )

    new_parser.add_argument(
        "-o",
        "--output-dir",
        type=str,
        required=False,
        help="specify the output directory for the new table function, default is `./<function_name>`",
    )


def parse_sys_args(sys_args):
    parser = _eggfarm_parser()
    sub_parsers = parser.add_subparsers(
        help=f"eggfarm generates python table function skeleton, version {EGGFARM_VERSION}",
        dest="command",
        required=True,
    )
    _new_command_parser(sub_parsers)
    args = parser.parse_args(sys_args)
    return vars(args)


def main():
    args = parse_sys_args(sys.argv[1:])
    sub_command = args["command"]

    actions = {
        "new": generate_skeleton,
    }
    actions[sub_command](args)
